==> fispro_3.4.orig.tar.gz: fispro-3.4/cpp/fisopt/sifopt.h <==
//***********************************************************************
//
//
//
//
// Author(s) : Serge GUILLAUME, Brigitte CHARNOMORDIC 
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//**********************************************************************
#ifndef SIFOPT_H
#define SIFOPT_H
#include "algo.h"
#include "fis.h"
//! Global FisEval function\n  Get a pointer to sifopt instance in conteneur and call sifopt::Eval
double FisEval( void *  , int  *  , double * ,int ,conteneur *);
/*!
 * \class sifopt
 * \brief this class inherit from algo class, but is specific to FIS.
 */

class sifopt : public algo  {
  sifopt& operator=(const sifopt&);
  sifopt(const sifopt&);
public:
//! Unused constructor.
      sifopt();
/*!
 * \param pS The adress of a fis instance
 * \param pMissClass a pointer to the Miss Classed array
 * \param pDouble a pointer to the Samples Data
 * \param nbrow the number of samples
 * \param Sortie the number of the output to optimize
 * \return an sifopt instance adress ( as a constructor usualy do )
This constructor calls algo constructor with the following params an Solis Wetts method ( 1 ) an size of ( 20 ) for conteneur. It read the Fis data and create his own data.
*/
      sifopt ( FIS     * pS ,int *& pMissClass , double **pDouble,int nbrow ,int Sortie  );
     ~sifopt(); //!< Free memory ...
      FIS     * InitialFis; //!< Initial value of the fis to optimise
      FIS     * InitialNormedFis; //!< Initial value of the normalized fis to optimise
      FIS     * tmpFis; //!< Current value of the fis
      vkey       PFFIn; //!< Boolean Key that tell for each Ins if they are "Partition Floue Forte". 
      int        PFFOut; //!< A boolean that tell if the Out that we currently use is a "Partition Floue Forte".
//! Key passed for Input
      vkey     KeyIn;
//! Key passed for Output
      vkey     KeyOut;
//! Key passed for Rules
      vkey     KeyRule;
//! reel size of double array
      int SizeIn,SizeOut;
//! initial Key ( given in command line ) to build other key
      vkey     InitKey;
//! The highest key for the fis
      unsigned long int MaxKey;
//! used not to call fis members functions to get type
      int     * MfType;
//! tells if the algorythm class members are initialized  
      int       IsInited;
//! tells if the algorithm found a solution 
      int       Reponse;
//! tells if the algorythm manage input or output
      int       IsInput;
//! tells is we will have to optimise the ouput
      int       FuzzyOutput;
//! if the output is crisp the conclusions are permutation of the existings ones
      int       LimitedVocabulary;
//! not used anymore ...
      int       CardVocabulary;

//! store Centers values of the inputs to check the constraints The Dim of this array is nbEntree + 1 for the output ( if it's a Fuzzy one ).
      double ** InputCenters;
//! store Centers values Max allowed when CNear is activate 
      double ** InputCentersMax;
//! store Centers values Min allowed when CNear is activate 
      double ** InputCentersMin;
//! Tell is the CNear constraint is activated
      int CNearActivate;
//! Tell is the CNear constraint array is inited
      int CNearInitIn;
//! Tell is the CNear constraint array is inited
      int CNearInitOut;
//! store an boolean to tell if we can modify the value of a center if we chose constraints that modify the centers value instead of considering a 'FAIL' case.
      int    ** InputCentersUpdatable;
//! store the MF type to perform a reading of the Entry by a pType++ instead of calling FIS members classes.
      int    ** InputCentersType;
//! store the MF parameters number instead of calling FIS members classes.
      int    ** InputCentersNbPar;
//! store The entry max size ( not PFF how many parameters ).
      int     * InputCentersLenght;
//! store The number of MF in an Entry
      int     * InputNbMf;
//! store The number of MF in the current Output.
      int       OutputNbMf;
//! store maximun possible size for the double Array of the Algorythm
      int       SumCentersLenght;
//! The number of inputs      
      int       nbEntree;
      int       nbRow;
 public:
// the output number we optimise
      int       numOutput;
// the number of rules
      int       NbRule;
      int       KeyRuleLenght;
int ** pResClassif;
double ** Exemples;

//! Value of the fis with the lowest evaluation
      FIS     * Solution;
int    init(void);
 void Update(int * ResClassif,double ** &Exemples,int &nout, int &nbrow, int &cnear, double &InitCoverage, double &cov); 
 
 void UpdatefEvalArgs(double ** &exemples, int & nbrow, double & couverture, double & maxerror, 
		      int * &ResClassif, double & museuil, double *& lab );
 int    launchEntry(double & gain, bool wordless);
double Eval( void *  Item  , vkey *  Key , double *Array ,int sizeArray );
int    DoubleToObject ( double * , int  , vkey *  ,void *  );
void   ObjectToDouble( void * , vkey * , double * );
//! used to store initial key who will be transformed in the init part
void    setKeyInit( char * );
//! used to store initial key who will be transformed in the init part
void    setKeyInit( vkey * );
//!  used to generate a "full key" ( all boolean are 1 ) whith the current fis and the output passed in argument
vkey    *keyallopt( int , FIS * );
//!  use to set the key to optimize an input
vkey * keysetIN( FIS * F , int nIn , int indPFF );
//!  use to set the key to optimize an MF of an input
vkey * keysetMFIN( FIS * F , int nIn , int indPFF , int nMF );
//!  use to set the key to optimize an output
vkey * keysetOUT( FIS * F , int nOut , int indPFF );
//!  use to set the key to optimize an Mf of an output
vkey * keysetMFOUT( FIS * F , int nOut , int indPFF, int nMF );
//!  use to set the key to optimize a rule
vkey * keysetRULE( FIS * F , int nOut ,int nRule );
int testConstraints( double *  , int  , vkey *  );
int CSort( int , int );
int CSortCenter( int , int );
int CDomain ( int , int);
int CNear ( int,int );

  // Added by SG - March 16th, 2010
 double InitCov, CovLoss; 
// Initial coverage index and authorized loss of coverage
};
#endif

==> fispro_3.6.orig.tar.gz: fispro-3.6/cpp/fisopt/sifopt.h <==
//***********************************************************************
//
//
//
//
// Author(s) : Serge GUILLAUME, Brigitte CHARNOMORDIC 
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//**********************************************************************
#ifndef SIFOPT_H
#define SIFOPT_H
#include "algo.h"
#include "fis.h"
//! Global FisEval function\n  Get a pointer to sifopt instance in conteneur and call sifopt::Eval
double FisEval( void *  , int  *  , double * ,int ,conteneur *);
/*!
 * \class sifopt
 * \brief this class inherit from algo class, but is specific to FIS.
 */

struct parstruct
{
  int maxite;
  double gauss;
  int maxcons;
  int maxfail;
  double epsiloncenter;
  double l1;
  double l2;
  double l3;
  
};

class sifopt : public algo  {
  sifopt& operator=(const sifopt&);
  sifopt(const sifopt&);
public:
//! Unused constructor.
      sifopt();
/*!
 * \param pS The adress of a fis instance
 * \param pMissClass a pointer to the Miss Classed array
 * \param pDouble a pointer to the Samples Data
 * \param nbrow the number of samples
 * \param Sortie the number of the output to optimize
 * \return an sifopt instance adress ( as a constructor usualy do )
This constructor calls algo constructor with the following params an Solis Wetts method ( 1 ) an size of ( 20 ) for conteneur. It read the Fis data and create his own data.
*/
      sifopt ( FIS     * pS ,int *& pMissClass , double **pDouble,int nbrow ,int Sortie,double nearConstraint=0.001  );
     ~sifopt(); //!< Free memory ...
      FIS     * InitialFis; //!< Initial value of the fis to optimise
      FIS     * InitialNormedFis; //!< Initial value of the normalized fis to optimise
      FIS     * tmpFis; //!< Current value of the fis
      vkey       PFFIn; //!< Boolean Key that tell for each Ins if they are "Partition Floue Forte". 
      int        PFFOut; //!< A boolean that tell if the Out that we currently use is a "Partition Floue Forte".
//! Key passed for Input
      vkey     KeyIn;
//! Key passed for Output
      vkey     KeyOut;
//! Key passed for Rules
      vkey     KeyRule;
//! reel size of double array
      int SizeIn,SizeOut;
//! initial Key ( given in command line ) to build other key
      vkey     InitKey;
//! The highest key for the fis
      unsigned long int MaxKey;
//! used not to call fis members functions to get type
      int     * MfType;
//! tells if the algorythm class members are initialized  
      int       IsInited;
//! tells if the algorithm found a solution 
      int       Reponse;
//! tells if the algorythm manage input or output
      int       IsInput;
//! tells is we will have to optimise the ouput
      int       FuzzyOutput;
//! if the output is crisp the conclusions are permutation of the existings ones
      int       LimitedVocabulary;
//! not used anymore ...
      int       CardVocabulary;

//! store Centers values of the inputs to check the constraints The Dim of this array is nbEntree + 1 for the output ( if it's a Fuzzy one ).
      double ** InputCenters;
//! store Centers values Max allowed when CNear is activate 
      double ** InputCentersMax;
//! store Centers values Min allowed when CNear is activate 
      double ** InputCentersMin;
//! Tell is the CNear constraint is activated
      int CNearActivate;
//! Tell is the CNear constraint array is inited
      int CNearInitIn;
//! Tell is the CNear constraint array is inited
      int CNearInitOut;
//! Feb 2013 to implement nearConstraint as a real value between MF centers
      double NearConstraint;
//! store an boolean to tell if we can modify the value of a center if we chose constraints that modify the centers value instead of considering a 'FAIL' case.
      int    ** InputCentersUpdatable;
//! store the MF type to perform a reading of the Entry by a pType++ instead of calling FIS members classes.
      int    ** InputCentersType;
//! store the MF parameters number instead of calling FIS members classes.
      int    ** InputCentersNbPar;
//! store The entry max size ( not PFF how many parameters ).
      int     * InputCentersLenght;
//! store The number of MF in an Entry
      int     * InputNbMf;
//! store The number of MF in the current Output.
      int       OutputNbMf;
//! store maximun possible size for the double Array of the Algorythm
      int       SumCentersLenght;
//! The number of inputs      
      int       nbEntree;
      int       nbRow;
//! Index of MF char. pts that should be sorted and satisfy a min. distance constraint-bch Feb 13, 2013
      int    * MFptsInd;
 public:
// the output number we optimise
      int       numOutput;
// the number of rules
      int       NbRule;
      int       KeyRuleLenght;
int ** pResClassif;
double ** Exemples;

//! Value of the fis with the lowest evaluation
      FIS     * Solution;
int    init(void);
 void Update(int * ResClassif,double ** &Exemples,int &nout, int &nbrow, int &cnear, double &InitCoverage, double &cov); 
 
 void UpdatefEvalArgs(double ** &exemples, int & nbrow, double & couverture, double & maxerror, 
		      int * &ResClassif, double & museuil, double *& lab );
 int    launchEntry(double & gain, bool wordless);
double Eval( void *  Item  , vkey *  Key , double *Array ,int sizeArray );
int    DoubleToObject ( double * , int  , vkey *  ,void *  );
void   ObjectToDouble( void * , vkey * , double * );
//! used to store initial key who will be transformed in the init part
void    setKeyInit( char * );
//! used to store initial key who will be transformed in the init part
void    setKeyInit( vkey * );
//!  used to generate a "full key" ( all boolean are 1 ) whith the current fis and the output passed in argument
vkey    *keyallopt( int , FIS * );
//!  use to set the key to optimize an input
vkey * keysetIN( FIS * F , int nIn , int indPFF );
//!  use to set the key to optimize an MF of an input
vkey * keysetMFIN( FIS * F , int nIn , int indPFF , int nMF );
//!  use to set the key to optimize an output
vkey * keysetOUT( FIS * F , int nOut , int indPFF );
//!  use to set the key to optimize an Mf of an output
vkey * keysetMFOUT( FIS * F , int nOut , int indPFF, int nMF );
//!  use to set the key to optimize a rule
vkey * keysetRULE( FIS * F , int nOut ,int nRule );
int oldtestConstraints( double *  , int  , vkey *  );
int testConstraints( double *  , int  );
int oldCSort( int , int );
int oldCSortCenter( int , int );
int CSort( int , int );
int CSortCenter( int , int );
int CDomain ( int , int);
int CNear ( int,int );
 void PrintNC(){printf("NearConstraint=%f\n",NearConstraint);}

  // Added by SG - March 16th, 2010
 double InitCov, CovLoss; 
// Initial coverage index and authorized loss of coverage
};
#endif
